      SUBROUTINE TURNPRO ( T, TB, TC, ANGACC, NB, NC, 
     .                     ROOM, NOUT, TRNTIM, TRNROT ) 
C
C     Generate a turn orientation profile time series. 
C
C     Jeff Bytof      26-April-2001
C
C     Inputs
C     ------
C     T      = Total duration of turn in seconds.
C     TB     = Duration of spinup or spindown in seconds.
C     TC     = Duration of coast in seconds.
C     ANGACC = Constant angular acceleration vector at spinup.
C     NB     = Number of points returned for spinup/down phases. 
C     NC     = Number of points returned for coast phase.
C     ROOM   = Dimension of arrays in calling program.
C
C     Outputs
C     -------
C     NOUT   = Number of returned orientations.
C     TRNTIM = Orientation time array, in seconds.
C     TRNROT = Orientation array. 
C

      IMPLICIT        NONE


      DOUBLE PRECISION     T
      DOUBLE PRECISION     TB
      DOUBLE PRECISION     TC
      DOUBLE PRECISION     ANGACC  ( 3 )  

      INTEGER              NB
      INTEGER              NC
      INTEGER              NOUT
      INTEGER              ROOM 

      DOUBLE PRECISION     TRNTIM     ( ROOM )
      DOUBLE PRECISION     TRNROT     ( 3, 3, ROOM )

C
C     Local Variables 
C

      DOUBLE PRECISION     AV      ( 3 )  
      DOUBLE PRECISION     DTB 
      DOUBLE PRECISION     DTC 
      DOUBLE PRECISION     INFROT  ( 3, 3 )
      DOUBLE PRECISION     ROT     ( 3, 3 )
      DOUBLE PRECISION     TBASE 

      INTEGER              I
      INTEGER              I1
      INTEGER              I2
      INTEGER              I3
      INTEGER              J 
      INTEGER              K 


C
C     Initialize orientation matrix and angular velocity.
C
      
      CALL IDENT ( ROT ) 
 
      I1 = 1 
      I2 = 2 
      I3 = 3 

      DO I = 1, 3
         AV(I) = 0.D0
      END DO

C
C     Profile spinup orientations.
C

      DTB = TB / DBLE(NB-1) 
 
      NOUT = 1

      TBASE        = 0.D0
      TRNTIM(NOUT) = TBASE 

      DO J = 1, 3
         DO K = 1, 3
            TRNROT(J,K,NOUT) = ROT(J,K)
         END DO
      END DO

      DO I = 1, NB-1
 
         DO J = 1, 3
            AV(J) = AV(J) + ANGACC(J)*DTB
         END DO

         WRITE(*,*) 'av = ', AV
 

         CALL EUL2M  ( AV(3)*DTB, AV(2)*DTB, AV(1)*DTB, 
     .                        I3,        I2,        I1, INFROT )

C
C     Apply infinitesimal rotation.
C

         CALL MXM  ( INFROT, ROT, ROT )

         NOUT = NOUT + 1

         IF ( NOUT .GT. ROOM ) THEN
            WRITE(*,*) 'TURNPRO: nout > room'
            WRITE(*,*) 'nout, room = ', NOUT, ROOM
         END IF


         TRNTIM(NOUT) = DBLE(I)*DTB 

         DO J = 1, 3
            DO K = 1, 3
               TRNROT(J,K,NOUT) = ROT(J,K)
            END DO
         END DO

      END DO

C
C     Profile coast orientations.
C

      DTC = TC / DBLE(NC-1) 

      TBASE = TB
 
      DO I = 1, NC-1

         WRITE(*,*) 'av = ', AV
C
C     Angular velocity is constant.
C
 
         CALL EUL2M  ( AV(3)*DTC, AV(2)*DTC, AV(1)*DTC, 
     .                        I3,        I2,         I1, INFROT )

C
C     Apply infinitesimal rotation.
C

         CALL MXM  ( INFROT, ROT, ROT )
 
         NOUT = NOUT + 1

         IF ( NOUT .GT. ROOM ) THEN
            WRITE(*,*) 'TURNPRO: nout > room'
            WRITE(*,*) 'nout, room = ', NOUT, ROOM
         END IF

         TRNTIM(NOUT) = TBASE + DBLE(I)*DTC 

         DO J = 1, 3
            DO K = 1, 3
               TRNROT(J,K,NOUT) = ROT(J,K)
            END DO
         END DO

      END DO

C
C     Profile spindown orientations.
C

      DTB   = TB / DBLE(NB-1) 
      TBASE = TB + TC
 
      DO I = 1, NB-1

C
C     Reverse the angular acceleration.
C

         DO J = 1, 3
            AV(J) = AV(J) - ANGACC(J)*DTB
         END DO

         WRITE(*,*) 'av = ', AV

         CALL EUL2M  ( AV(3)*DTB, AV(2)*DTB, AV(1)*DTB, 
     .                        I3,        I2,        I1, INFROT )

C
C     Apply infinitesimal rotation.
C

         CALL MXM  ( INFROT, ROT, ROT )

         NOUT = NOUT + 1

         IF ( NOUT .GT. ROOM ) THEN
            WRITE(*,*) 'TURNPRO: nout > room'
            WRITE(*,*) 'nout, room = ', NOUT, ROOM
         END IF

         TRNTIM(NOUT) = TBASE + DBLE(I)*DTB 

         DO J = 1, 3
            DO K = 1, 3
               TRNROT(J,K,NOUT) = ROT(J,K)
            END DO
         END DO

      END DO

      RETURN
      END
